package com.contextspace.objectdb_example.model;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OrderBy;
import javax.persistence.Transient;

@Entity
public class Attribute extends IDHolder {
	
    private String name;
    private String displayName;
    
    private List<AttributeValue> attributeValues = new LinkedList<AttributeValue>();
    
    private boolean encrypted = false;
    
    /**
     * Constructor 
     * @param name
     * @param values
     */
    public Attribute(String name, List<Object> values) {
        this.name = name;
        for(int i=0; i < values.size();i++) {
        	this.attributeValues.add(new AttributeValue(i, values.get(i)));       	
        }
    }
    
    /**
     * Constructor 
     * @param name
     * @param values
     */
    public Attribute(String name, Object value) {
        this.name = name;
        this.attributeValues.add(new AttributeValue(this.attributeValues.size(), value));
    }
    
    /**
     * Constructor 
     * @param nam
     */
    public Attribute(String name) {
        this.name = name;
    }
    
    /**
     * Default constructor
     */
    public Attribute() {
    }
    
    /**
     * @return the name
     */
    public String getName() {
        return name;
    }
    /**
     * @param name the name to set
     */
    public void setName(String name) {
        this.name = name;
    }
    

    @OneToMany(cascade=CascadeType.ALL, fetch = FetchType.EAGER)
    @OrderBy("orderIndex ASC")
    @JoinColumn(name="ATTR_ID")
    public List<AttributeValue> getAttributeValues() {
    	return attributeValues;
    }
    
    public void setAttributeValues(List<AttributeValue> attributeValues) {
		this.attributeValues = attributeValues;
	}

	/**
     * @return the values
     */
    public List<Object> getValues() {

    	List<Object> valuesCopy = new ArrayList<Object>();
    	for(int i=0; i < attributeValues.size();i++){
    		String value = attributeValues.get(i).getValue();
    		String type = attributeValues.get(i).getType();
    		if(type.equals(String.class.getName())) {
    			valuesCopy.add(i,value);
    		} else if(type.equals(Boolean.class.getName())) {
    			valuesCopy.add(i,new Boolean(value));
    		} else if(type.equals(Integer.class.getName())) {
    			valuesCopy.add(i, new Integer(value));
    		} else if(type.equals(Long.class.getName())) {
    			valuesCopy.add(i, new Long(value));
    		} else {
    			try {
    				Object obj = Class.forName(type).getConstructor(String.class).newInstance(value);
    				valuesCopy.add(i, obj);
    			} catch (SecurityException e) {
    				e.printStackTrace();
    			} catch (NoSuchMethodException e) {
    				e.printStackTrace();
    			} catch (IllegalArgumentException e) {
    				e.printStackTrace();
    			} catch (InstantiationException e) {
    				e.printStackTrace();
    			} catch (IllegalAccessException e) {
    				e.printStackTrace();
    			} catch (InvocationTargetException e) {
    				e.printStackTrace();
    			} catch (ClassNotFoundException e) {
    				e.printStackTrace();
    			}
    		}
    	}
        return valuesCopy;
    }
    
    /**
     * @param values the values to set
     */
    public void setValues(List<Object> values) {
    	this.attributeValues.clear();
    	for(int i=0; i < values.size();i++) {
        	this.attributeValues.add(new AttributeValue(this.attributeValues.size(),values.get(i)));
        }
    }
    
    /**
     * Add value to end of list
     * @param value
     */
    public void addValue(Object value) {
        this.attributeValues.add(new AttributeValue(this.attributeValues.size(), value));
    }
    
    /**
     * Remove value from list
     * @param value
     */
    public void removeValue(Object value) {
        int indexToRemove = 0;
        boolean found = false;
        for(int i=0; i<attributeValues.size(); i++) {
            AttributeValue attrValue = attributeValues.get(i);
            //System.out.println("Comparing remove value " + value + " with array[" + i + "]: " + attrValue.getValue());
            if(attrValue.getValue().equals(value)) {
                indexToRemove = i;
                found = true;
            }
            else if(found) {
                attrValue.setOrderIndex(attrValue.getOrderIndex() - 1);
            }
        }
        attributeValues.remove(indexToRemove);
    }

    /**
     * @return the displayName
     */
    public String getDisplayName() {
        return displayName;
    }

    /**
     * @param displayName the displayName to set
     */
    public void setDisplayName(String displayName) {
        this.displayName = displayName;
    }

    /**
     * @return the encrypted
     */
    public boolean isEncrypted() {
        return encrypted;
    }

    /**
     * @param encrypted the encrypted to set
     */
    public void setEncrypted(boolean encrypted) {
        this.encrypted = encrypted;
    }
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("<attribute name=\"").append(name).append("\" displayName=\"").append(displayName).append("\" encrypted=\"").append(encrypted).append("\" values=\"");
        if(!attributeValues.isEmpty()) {
            for(int i=0; i<attributeValues.size(); i++) {
                if(i>0) {
                    buf.append(":");
                }
                buf.append(attributeValues.get(i));
            }
        }
        buf.append("\"");
        if(getId() != 0) {
            buf.append(" id=\"").append(getId()).append("\"");
        }
        
        buf.append("/>\n");
        return buf.toString();
    }   
}
