package com.contextspace.objectdb_example.model;

import javax.jdo.annotations.Index;
import javax.persistence.Basic;
import javax.persistence.Entity;

@Entity
public class AttributeValue extends IDHolder implements Comparable<AttributeValue> {
	private String value;
	private String type;
	private int orderIndex = 0;

	public AttributeValue() {}
	
	public AttributeValue(Object obj) {
		value = obj.toString();
		type = obj.getClass().getName();
	}
	
	public AttributeValue(int orderIndex, Object obj) {
		value = obj.toString();
		type = obj.getClass().getName();
		this.orderIndex = orderIndex;
	}
	
	@Index
	@Basic
	public String getValue() {
		return value;
	}

	public void setValue(String value) {
		if(value != null) {
			this.value = value.toString();
			type = value.getClass().getName();
		}
	}
	
	public void setValue(Object value) {
		if(value != null) {
			this.value = value.toString();
			type = value.getClass().getName();
		}
	}
	
	@Override
	public String toString() {
		return value;
	}
	
	@Override
	 public boolean equals(Object obj) {
		if (obj == null || (getClass() != obj.getClass())) {
            return false;
        }
        if (this == obj) {
            return true;
        }
        
        AttributeValue other = (AttributeValue) obj;
        if (value == null || other.getValue() == null) {
                return false;                
        } else if ((value.equals(other.getValue()) || (getId() == other.getId()))) {
            return true;
        }
        return false;
	}

	@Basic
	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public int getOrderIndex() {
		return orderIndex;
	}

	public void setOrderIndex(int orderIndex) {
		this.orderIndex = orderIndex;
	}

	public int compareTo(AttributeValue av) {
        // "this" was created before "av"
        if(orderIndex < av.getOrderIndex()) {
            return -1;
        } else if(orderIndex > av.getOrderIndex()) { // "this" was created after "av"
            return 1;
        } else { // Both are equal. This should never happened as there is a Unique constraint on the ID field
            return 0;
        }
    }

}
