package com.contextspace.objectdb_example.model;

import java.util.ArrayList;
import java.util.List;

import javax.jdo.annotations.Index;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;

import javax.persistence.Table;

@Entity
@Table
public class CDI extends IDHolder {
    private String cdiId = null;
    private List<Attribute> attributeList = new ArrayList<Attribute>();
    
    public CDI() {
    }
    
    /**
     * @return the cdiId
     */
    @Index(unique="true")
    @Column(nullable=false,unique=true) // this is only used by ORM for creating the tables with constraints 
    public String getCdiId() {
        return cdiId;
    }
    /**
     * @param cdiId the cdiId to set
     */
    public void setCdiId(String cdiId) {
        this.cdiId = cdiId;
    }
    
 
    public Attribute getAttribute(String name) {
        for(int i=0; i<attributeList.size(); i++) {
            Attribute attr = attributeList.get(i);
            if(attr.getName().equals(name)) {
                return attr;
            }
        }
        return null;
    }
   
    public void addAttribute(Attribute attribute) {
        attributeList.add(attribute);
    }

    public void removeAttribute(String name) {
    
        for(int i = 0;i < attributeList.size(); i++) {
            if(attributeList.get(i).getName().equals(name)) {
                attributeList.remove(attributeList.get(i));
            }
        }   
    }
        
    /**
     * @param attributeToRemove
     */
    public void removeAttribute(Attribute attributeToRemove) {
        String name = attributeToRemove.getName();
        removeAttribute(name);
    }
    
    /**
     * @param attributeToReplace
     */
    public void replaceAttribute(Attribute attributeToReplace) {
        String name = attributeToReplace.getName();

        for(int i = 0;i < attributeList.size(); i++) {
            if(attributeList.get(i).getName().equals(name)) {
                attributeList.set(i, attributeToReplace);
            }
        }   
    }
    
    /**
     * Get the attributes as a list
     * @return
     */
    @OneToMany(cascade=CascadeType.ALL, fetch = FetchType.EAGER, orphanRemoval=true)
    @JoinColumn(name="OWNER_ID")
    public List<Attribute> getAttributeList() {
        return attributeList;
    }
    
    /**
     * Set the attribute map from a list
     * @param attrList
     */
    public void setAttributeList(List<Attribute> attrList) {
        attributeList = attrList;
    }
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("<cdi cdiId=\"").append(cdiId).append("\">\n");
        if(!attributeList.isEmpty()) {
            buf.append("<attributes>\n");
            for(Attribute attr: attributeList) {
                buf.append(attr.toString());
            }
            buf.append("</attributes>\n");
        }
        buf.append("</cdi>\n");
        return buf.toString();
    }
}
